package com.example.wmarket.ViewModelsPack

import android.content.Context
import androidx.compose.runtime.State
import androidx.compose.runtime.mutableStateOf
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.wmarket.data.model.UserState
import com.example.wmarket.data.network.WMarketClient.client
import com.example.wmarket.utils.SharedPreferenceHelper
import io.github.jan.supabase.gotrue.auth
import io.github.jan.supabase.gotrue.providers.builtin.Email
import kotlinx.coroutines.launch

class SupabaseAuthViewModel: ViewModel() {
    private val _userState = mutableStateOf<UserState>(UserState.Loading)
    val userState: State<UserState> = _userState

    fun signUp(
        context: Context,
        userEmail: String,
        userPassword: String
    ){
        viewModelScope.launch {
            try {
                client.auth.signUpWith(Email){
                    email = userEmail
                    password = userPassword
                }
                saveToken(context)

                _userState.value = UserState.Success("Registered user successfully!")
            } catch (e: Exception){
                _userState.value = UserState.Error("Error ${e.message}")
            }
        }
    }

    private fun saveToken(context: Context){
        viewModelScope.launch {
            val accessToken = client.auth.currentAccessTokenOrNull() ?: ""
            val sharedPref = SharedPreferenceHelper(context)
            sharedPref.saveStringData("accessToken", accessToken)
        }
    }

    private fun getToken(context: Context): String?{
        val sharedPref = SharedPreferenceHelper(context)
        return sharedPref.getStringData("accessToken")
    }

    fun singIn(
        context: Context,
        userEmail: String,
        userPassword: String,
    ){
        viewModelScope.launch {
            try {
                client.auth.signInWith(Email){
                    email = userEmail
                    password = userPassword
                }
                saveToken(context)
                _userState.value = UserState.Success("SingUp successfully!")

            }catch (e: Exception){
                _userState.value = UserState.Error("Error ${e.message}")
            }
        }
    }

    fun singOut(context: Context){
        val sharedPref = SharedPreferenceHelper(context)
        viewModelScope.launch {
            try {
                client.auth.signOut()
                sharedPref.clearPreferences()
                _userState.value = UserState.Success("SingOut successfully!")
            }catch (e: Exception){
                _userState.value = UserState.Error("Error ${e.message}")

            }
        }
    }

    fun isUserSingIn(
        context: Context
    ){
        viewModelScope.launch {
            try {
                val token = getToken(context)
                if(token.isNullOrEmpty()){
                    _userState.value = UserState.Error("Пользователь не вошёл в систему!")
                } else{
                    client.auth.retrieveUser(token)
                    client.auth.refreshCurrentSession()
                    saveToken(context)
                    _userState.value = UserState.Success("Пользователь уже вошёл в систему!")
                }

            } catch (e: Exception){
                _userState.value = UserState.Error("Error ${e.message}")
            }
        }
    }

}